var fs = require('fs');
var path = require('path');
var ig = require('imagemagick');
var colors = require('colors');
var _ = require('underscore');
var Q = require('q');
var nodeFs = require('node-fs');

/**
 * @var {Object} settings - names of the confix file and of the icon image
 * TODO: add option to get these values as CLI params
 */
var settings = {};

/**
 * @var {Object} console utils
 */
var display = {};
display.success = function (str) {
  str = '✓  '.green + str;
  console.log('  ' + str);
};
display.error = function (str) {
  str = '✗  '.red + str;
  console.log('  ' + str);
};
display.header = function (str) {
  console.log('');
  console.log(' ' + str.cyan.underline);
  console.log('');
};

/**
 * Check which platforms are added to the project and return their icon names and sized
 *
 * @param  {String} projectName
 * @return {Promise} resolves with an array of platforms
 */
function getPlatformIcons() {
  var deferred = Q.defer();
  var platforms = settings.ICON_PLATFORMS;

  deferred.resolve(platforms);
  return deferred.promise;
};

/**
 * Check which platforms are added to the project and return their splash screen names and sizes
 *
 * @param  {String} projectName
 * @return {Promise} resolves with an array of platforms
 */
function getPlatformSplashs() {
  var deferred = Q.defer();
  var platforms = settings.SPLASH_PLATFORMS

  deferred.resolve(platforms);
  return deferred.promise;
};

/**
 * Resizes and creates a new icon in the platform's folder.
 *
 * @param  {Object} platform
 * @param  {Object} icon
 * @return {Promise}
 */
function generateIcon(platform, icon) {
  var deferred = Q.defer();
  try {
    var filePath = path.join(platform.iconsPath, icon.name);
    var filedirName = path.dirname(filePath);
    if (!fs.existsSync(filedirName)) {
      nodeFs.mkdirSync(filedirName, '0777', true);
    }
    var srcPath = settings.ICON_FILE;
    var platformIconPath = path.join(path.dirname(srcPath), path.basename(srcPath));
    if (fs.existsSync(platformIconPath)) {
      srcPath = platformIconPath;
    }
    ig.resize({
      srcPath: srcPath,
      dstPath: filePath,
      quality: 1,
      format: icon.name.replace(/.*\.(\w+)$/i, '$1').toLowerCase(),
      width: icon.size,
      height: icon.size,
    }, function (err, stdout, stderr) {
      if (err) {
        deferred.reject(err);
      } else {
        deferred.resolve();
        display.success(icon.name + ' created');
      }
    });
  } catch (err) {
    deferred.reject(err);
  }
  return deferred.promise;
};

/**
 * Generates icons based on the platform object
 *
 * @param  {Object} platform
 * @return {Promise}
 */
function generateIconsForPlatform(platform) {
  var deferred = Q.defer();
  display.header('Generating Icons for ' + platform.name);
  var all = [];
  var icons = platform.icons;
  icons.forEach(function (icon) {
    all.push(generateIcon(platform, icon));
  });
  Q.all(all).then(function () {
    deferred.resolve();
  }).catch(function (err) {
    console.log(err);
  });
  return deferred.promise;
};

/**
 * Goes over all the platforms and triggers icon generation
 *
 * @param  {Array} platforms
 * @return {Promise}
 */
function generateIcons(platforms) {
  var deferred = Q.defer();
  var sequence = Q();
  var all = [];
  _(platforms).where({
    isAdded: true
  }).forEach(function (platform) {
    sequence = sequence.then(function () {
      return generateIconsForPlatform(platform);
    });
    all.push(sequence);
  });
  Q.all(all).then(function () {
    deferred.resolve();
  });
  return deferred.promise;
};


/**
 * Checks if a valid icon file exists
 *
 * @return {Promise} resolves if exists, rejects otherwise
 */
function validIconExists() {
  var deferred = Q.defer();
  fs.exists(settings.ICON_FILE, function (exists) {
    if (exists) {
      display.success(settings.ICON_FILE + ' exists');
      deferred.resolve(true);
    } else {
      display.error(settings.ICON_FILE + ' does not exist in the root folder');
      deferred.resolve(false);
    }
  });
  return deferred.promise;
};


/**
 * Crops and creates a new splash in the platform's folder.
 *
 * @param  {Object} platform
 * @param  {Object} splash
 * @return {Promise}
 */
function generateSplash(platform, splash) {
  var deferred = Q.defer();
  try {
    var filePath = path.join(platform.splashPath, splash.name);
    var filedirName = path.dirname(filePath);
    if (!fs.existsSync(filedirName)) {
      nodeFs.mkdirSync(filedirName, '0777', true);
    }
    var srcPath = settings.SPLASH_FILE;
    var platformIconPath = path.join(path.dirname(srcPath), path.basename(srcPath));
    if (fs.existsSync(platformIconPath)) {
      srcPath = platformIconPath;
    }
    ig.crop({
      srcPath: srcPath,
      dstPath: filePath,
      quality: 1,
      format: splash.name.replace(/.*\.(\w+)$/i, '$1').toLowerCase(),
      width: splash.width,
      height: splash.height,
    }, function (err, stdout, stderr) {
      if (err) {
        deferred.reject(err);
      } else {
        deferred.resolve();
        display.success(splash.name + ' created');
      }
    });
  } catch (err) {
    deferred.reject(err);
  }
  return deferred.promise;
};

/**
 * Generates splash based on the platform object
 *
 * @param  {Object} platform
 * @return {Promise}
 */
function generateSplashForPlatform(platform) {
  var deferred = Q.defer();
  display.header('Generating splash screen for ' + platform.name);
  var all = [];
  var splashes = platform.splash;
  splashes.forEach(function (splash) {
    all.push(generateSplash(platform, splash));
  });
  Q.all(all).then(function () {
    deferred.resolve();
  }).catch(function (err) {
    console.log(err);
  });
  return deferred.promise;
};

/**
 * Goes over all the platforms and triggers splash screen generation
 *
 * @param  {Array} platforms
 * @return {Promise}
 */
function generateSplashes(platforms) {
  var deferred = Q.defer();
  var sequence = Q();
  var all = [];
  _(platforms).where({
    isAdded: true
  }).forEach(function (platform) {
    sequence = sequence.then(function () {
      return generateSplashForPlatform(platform);
    });
    all.push(sequence);
  });
  Q.all(all).then(function () {
    deferred.resolve();
  });
  return deferred.promise;
};
/**
 * Checks if a valid splash file exists
 *
 * @return {Promise} resolves if exists, rejects otherwise
 */
function validSplashExists() {
  var deferred = Q.defer();
  fs.exists(settings.SPLASH_FILE, function (exists) {
    if (exists) {
      display.success(settings.SPLASH_FILE + ' exists');
      deferred.resolve(true);
    } else {
      display.error(settings.SPLASH_FILE + ' does not exist in the root folder');
      deferred.resolve(false);
    }
  });
  return deferred.promise;
};

function generate(options) {
  settings = options;
  display.header('Checking Splash & Icon');
  return Q.all([validIconExists(), validSplashExists()])
    .then(function (results) {
      var hasIcon = results[0] === true;
      var hasSplash = results[1] === true;
      var promise;

      if (!hasIcon && !hasSplash) {
        //console.log(arguments);
        promise = Q.reject();
      }

      if (hasIcon) {
        promise = Q.when(promise)
          .then(getPlatformIcons)
          .then(generateIcons);
      }

      if (hasSplash) {
        promise = Q.when(promise)
          .then(getPlatformSplashs)
          .then(generateSplashes);
      }

      return Q.when(promise);
    })
    .catch(function (err) {
      if (err) {
        console.log(err);
      }
    }).then(function () {
      console.log('');
    });
}

module.exports = {
  generate: generate
};